#include <msp/gl/extensions/arb_enhanced_layouts.h>
#include <msp/gl/extensions/arb_explicit_attrib_location.h>
#include <msp/gl/extensions/arb_explicit_uniform_location.h>
#include <msp/gl/extensions/arb_gpu_shader5.h>
#include <msp/gl/extensions/arb_separate_shader_objects.h>
#include <msp/gl/extensions/arb_uniform_buffer_object.h>
#include <msp/gl/extensions/ext_gpu_shader4.h>
#include <msp/gl/extensions/ext_texture_array.h>
#include "features.h"

namespace Msp {
namespace GL {
namespace SL {

Features::Features():
	gl_api(OPENGL),
	arb_enhanced_layouts(false),
	arb_explicit_attrib_location(false),
	arb_explicit_uniform_location(false),
	arb_gpu_shader5(false),
	arb_separate_shader_objects(false),
	arb_uniform_buffer_object(false),
	ext_gpu_shader4(false),
	ext_texture_array(false),
	constant_id_range(0x80000000U)
{ }

Features Features::from_context()
{
	Features features;
	features.gl_api = get_gl_api();
	features.glsl_version = get_glsl_version();
	features.arb_enhanced_layouts = ARB_enhanced_layouts;
	features.arb_explicit_attrib_location = ARB_explicit_attrib_location;
	features.arb_explicit_uniform_location = ARB_explicit_uniform_location;
	features.arb_gpu_shader5 = ARB_gpu_shader5;
	features.arb_separate_shader_objects = ARB_separate_shader_objects;
	features.arb_uniform_buffer_object = ARB_uniform_buffer_object;
	features.ext_gpu_shader4 = EXT_gpu_shader4;
	features.ext_texture_array = EXT_texture_array;
	return features;
}

Features Features::all()
{
	Features features;
	features.gl_api = OPENGL;
	features.glsl_version = Version(4, 60);
	features.arb_enhanced_layouts = true;
	features.arb_explicit_attrib_location = true;
	features.arb_explicit_uniform_location = true;
	features.arb_gpu_shader5 = true;
	features.arb_separate_shader_objects = true;
	features.arb_uniform_buffer_object = true;
	features.ext_gpu_shader4 = true;
	features.ext_texture_array = true;
	return features;
}

} // namespace SL
} // namespace GL
} // namespace Msp
